@@ -7,14 +7,11 @@ use xds_api::pb::envoy::config::{
 };
 
 use crate::{
-    backend::{
-        Backend, LbPolicy, RingHashParams, SessionAffinity, SessionAffinityHashParam,
-        SessionAffinityHashParamType,
-    },
+    backend::{Backend, LbPolicy, RequestHashPolicy, RequestHasher, RingHashParams},
     error::{Error, ErrorContext},
     value_or_default,
     xds::ads_config_source,
-    BackendId, Target,
+    BackendId, Service,
 };
 
 impl Backend {
@@ -29,11 +26,11 @@ impl Backend {
 
         let discovery_type = cluster_discovery_type(cluster);
 
-        match &id.target {
+        match &id.service {
             // if this is supposed to be a DNS cluster, validate that the xDS
             // actually says it's a DNS cluster and that the discovery data
             // matches the name.
-            Target::Dns(dns) => {
+            Service::Dns(dns) => {
                 if discovery_type != Some(DiscoveryType::LogicalDns) {
                     return Err(Error::new_static("mismatched discovery type"))
                         .with_field("cluster_discovery_type");
@@ -52,7 +49,7 @@ impl Backend {
                 }
             }
             // kube clusters should use EDS over ADS, with a cluster_name matching cluster.name
-            Target::KubeService(_) => {
+            Service::Kube(_) => {
                 if discovery_type != Some(DiscoveryType::Eds) {
                     return Err(Error::new_static("mismatched discovery type"))
                         .with_field("cluster_discovery_type");
@@ -78,7 +75,7 @@ impl Backend {
         Ok(Backend { id, lb })
     }
 
-    pub fn to_xds_cluster(&self) -> xds_cluster::Cluster {
+    pub fn to_xds(&self) -> xds_cluster::Cluster {
         use xds_cluster::cluster::ClusterDiscoveryType;
         use xds_cluster::cluster::DiscoveryType;
         use xds_cluster::cluster::EdsClusterConfig;
@@ -88,8 +85,8 @@ impl Backend {
             None => (xds_cluster::cluster::LbPolicy::default(), None),
         };
 
-        let (cluster_discovery_type, eds_cluster_config, load_assignment) = match &self.id.target {
-            Target::Dns(dns) => {
+        let (cluster_discovery_type, eds_cluster_config, load_assignment) = match &self.id.service {
+            Service::Dns(dns) => {
                 let dtype = ClusterDiscoveryType::Type(DiscoveryType::LogicalDns.into());
                 let host_identifier = Some(xds_endpoint::lb_endpoint::HostIdentifier::Endpoint(
                     xds_endpoint::Endpoint {
@@ -110,7 +107,7 @@ impl Backend {
                 });
                 (dtype, None, load_assignment)
             }
-            Target::KubeService(_) => {
+            Service::Kube(_) => {
                 let cluster_discovery_type = ClusterDiscoveryType::Type(DiscoveryType::Eds.into());
                 let eds_cluster_config = Some(EdsClusterConfig {
                     eds_config: Some(crate::xds::ads_config_source()),
@@ -132,7 +129,12 @@ impl Backend {
         }
     }
 
-    pub fn to_xds_passthrough_route(&self) -> xds_route::RouteConfiguration {
+    /// Generate a RouteConfiguration that routes to this Cluster using its
+    /// hash_policy as RouteAction. This exists so we can guarantee that there's
+    /// at least one unique RouteConfiguration pointing at every cluster so the
+    /// client can deduce its hash policies that way.
+    #[doc(hidden)]
+    pub fn to_xds_lb_route_config(&self) -> xds_route::RouteConfiguration {
         use xds_route::route::Action;
         use xds_route::route_action::ClusterSpecifier;
         use xds_route::route_match::PathSpecifier;
@@ -158,13 +160,8 @@ impl Backend {
             ..Default::default()
         };
 
-        // NOTE: this is the only place where we use `backend_name` to name a
-        // RouteConfiguration.
-        //
-        // it's incorrect literally everywhere else, but should be done here to
-        // indicate that this is the passthrough route.
         xds_route::RouteConfiguration {
-            name: self.id.passthrough_route_name(),
+            name: self.id.lb_config_route_name(),
             virtual_hosts: vec![vhost],
             ..Default::default()
         }
@@ -325,30 +322,34 @@ impl LbPolicy {
 }
 
 #[inline]
-fn hash_policies(action: &xds_route::RouteAction) -> Result<Vec<SessionAffinityHashParam>, Error> {
+fn hash_policies(action: &xds_route::RouteAction) -> Result<Vec<RequestHashPolicy>, Error> {
     let res: Result<Vec<_>, Error> = action
         .hash_policy
         .iter()
         .enumerate()
-        .map(|(i, policy)| SessionAffinityHashParam::from_xds(policy).with_index(i))
+        .map(|(i, policy)| RequestHashPolicy::from_xds(policy).with_index(i))
         .collect();
 
     res.with_field("hash_policy")
 }
 
-impl SessionAffinityHashParam {
+impl RequestHashPolicy {
     pub(crate) fn to_xds(&self) -> xds_route::route_action::HashPolicy {
-        use xds_route::route_action::hash_policy::Header;
-        use xds_route::route_action::hash_policy::PolicySpecifier;
+        use xds_route::route_action::hash_policy::{Header, PolicySpecifier, QueryParameter};
 
-        match &self.matcher {
-            SessionAffinityHashParamType::Header { name } => xds_route::route_action::HashPolicy {
-                terminal: self.terminal,
-                policy_specifier: Some(PolicySpecifier::Header(Header {
-                    header_name: name.clone(),
-                    regex_rewrite: None,
-                })),
-            },
+        let policy_specifier = match &self.hasher {
+            RequestHasher::Header { name } => PolicySpecifier::Header(Header {
+                header_name: name.clone(),
+                regex_rewrite: None,
+            }),
+            RequestHasher::QueryParam { name } => {
+                PolicySpecifier::QueryParameter(QueryParameter { name: name.clone() })
+            }
+        };
+
+        xds_route::route_action::HashPolicy {
+            terminal: self.terminal,
+            policy_specifier: Some(policy_specifier),
         }
     }
 
@@ -358,10 +359,16 @@ impl SessionAffinityHashParam {
         match &xds.policy_specifier {
             Some(PolicySpecifier::Header(header)) => Ok(Self {
                 terminal: xds.terminal,
-                matcher: SessionAffinityHashParamType::Header {
+                hasher: RequestHasher::Header {
                     name: header.header_name.clone(),
                 },
             }),
+            Some(PolicySpecifier::QueryParameter(query)) => Ok(Self {
+                terminal: xds.terminal,
+                hasher: RequestHasher::QueryParam {
+                    name: query.name.clone(),
+                },
+            }),
             Some(_) => {
                 Err(Error::new_static("unsupported hash policy").with_field("policy_specifier"))
             }
@@ -370,77 +377,58 @@ impl SessionAffinityHashParam {
     }
 }
 
-impl SessionAffinity {
-    pub fn from_xds(
-        hash_policy: &[xds_route::route_action::HashPolicy],
-    ) -> Result<Option<Self>, Error> {
-        if hash_policy.is_empty() {
-            return Ok(None);
-        }
-
-        let hash_params = hash_policy
-            .iter()
-            .enumerate()
-            .map(|(i, h)| SessionAffinityHashParam::from_xds(h).with_index(i))
-            .collect::<Result<Vec<_>, _>>()?;
-
-        debug_assert!(!hash_params.is_empty(), "hash params must not be empty");
-        Ok(Some(Self { hash_params }))
-    }
-}
-
 #[cfg(test)]
 mod test {
     use super::*;
 
     #[test]
     fn test_unspecified_lb_roundtrips() {
-        let web = Target::kube_service("prod", "web").unwrap();
+        let web = Service::kube("prod", "web").unwrap();
 
         let backend = Backend {
-            id: web.into_backend(8891),
+            id: web.as_backend_id(8891),
             lb: LbPolicy::Unspecified,
         };
-        assert_eq!(
-            backend,
-            Backend::from_xds(&backend.to_xds_cluster(), None).unwrap(),
-        );
+        assert_eq!(backend, Backend::from_xds(&backend.to_xds(), None).unwrap(),);
         assert_eq!(backend.to_xds_hash_policies(), vec![]);
     }
 
     #[test]
     fn test_round_robin_lb_roundtrips() {
-        let web = Target::kube_service("prod", "web").unwrap();
+        let web = Service::kube("prod", "web").unwrap();
 
         let backend = Backend {
-            id: web.into_backend(7891),
+            id: web.as_backend_id(7891),
             lb: LbPolicy::RoundRobin,
         };
-        assert_eq!(
-            backend,
-            Backend::from_xds(&backend.to_xds_cluster(), None).unwrap(),
-        );
+        assert_eq!(backend, Backend::from_xds(&backend.to_xds(), None).unwrap(),);
         assert_eq!(backend.to_xds_hash_policies(), vec![]);
     }
 
     #[test]
     fn test_ringhash_roundtrip() {
-        let web = Target::kube_service("prod", "web").unwrap();
+        let web = Service::kube("prod", "web").unwrap();
 
         let backend = Backend {
-            id: web.into_backend(6666),
+            id: web.as_backend_id(6666),
             lb: LbPolicy::RingHash(RingHashParams {
                 min_ring_size: 1024,
                 hash_params: vec![
-                    SessionAffinityHashParam {
+                    RequestHashPolicy {
+                        terminal: true,
+                        hasher: RequestHasher::QueryParam {
+                            name: "q".to_string(),
+                        },
+                    },
+                    RequestHashPolicy {
                         terminal: false,
-                        matcher: SessionAffinityHashParamType::Header {
+                        hasher: RequestHasher::Header {
                             name: "x-user".to_string(),
                         },
                     },
-                    SessionAffinityHashParam {
-                        terminal: false,
-                        matcher: SessionAffinityHashParamType::Header {
+                    RequestHashPolicy {
+                        terminal: true,
+                        hasher: RequestHasher::Header {
                             name: "x-env".to_string(),
                         },
                     },
@@ -448,7 +436,7 @@ mod test {
             }),
         };
 
-        let cluster = backend.to_xds_cluster();
+        let cluster = backend.to_xds();
         let hash_policy = backend.to_xds_hash_policies();
 
         let parsed = Backend::from_xds(
@@ -463,23 +451,23 @@ mod test {
     }
 
     #[test]
-    fn test_passthrough_route_roundtrip() {
-        let web = Target::kube_service("prod", "web").unwrap();
+    fn test_lb_route_config_roundtrip() {
+        let web = Service::kube("prod", "web").unwrap();
 
         let backend = Backend {
-            id: web.into_backend(12321),
+            id: web.as_backend_id(12321),
             lb: LbPolicy::RingHash(RingHashParams {
                 min_ring_size: 1024,
                 hash_params: vec![
-                    SessionAffinityHashParam {
+                    RequestHashPolicy {
                         terminal: false,
-                        matcher: SessionAffinityHashParamType::Header {
+                        hasher: RequestHasher::Header {
                             name: "x-user".to_string(),
                         },
                     },
-                    SessionAffinityHashParam {
+                    RequestHashPolicy {
                         terminal: false,
-                        matcher: SessionAffinityHashParamType::Header {
+                        hasher: RequestHasher::Header {
                             name: "x-env".to_string(),
                         },
                     },
@@ -487,11 +475,11 @@ mod test {
             }),
         };
 
-        let cluster = backend.to_xds_cluster();
-        let passthrough_route = backend.to_xds_passthrough_route();
+        let cluster = backend.to_xds();
+        let lb_config_route = backend.to_xds_lb_route_config();
 
         let parsed = Backend::from_xds(&cluster, {
-            let vhost = passthrough_route.virtual_hosts.first().unwrap();
+            let vhost = lb_config_route.virtual_hosts.first().unwrap();
             let route = vhost.routes.first().unwrap();
             route.action.as_ref().map(|action| match action {
                 xds_route::route::Action::Route(action) => action,
